<h3>Activation Email</h3>

<p>Hello {{first_name}} {{last_name}}</p>

<p>Please go to activation page to finish your registration,
    your username is <b>{{username}}</b>
</p>

<a href="{{url}}">Follow this link to activate</a>

