{% extends "appbuilder/general/model/list.html" %}

{% block content %}

    Text on top of the page
    {{ super() }}
{% endblock %}

{% block list_search scoped %}
    Text before the search section
    {{ super() }}
{% endblock %}

{% block list_list scoped %}
        Text before the list
        {{ super() }}
{% endblock %}


